/**
 * This file is a dummy file whose sole purpose is to determine which set of
 * CFLAGS can be added. It is imperative that this always will compile
 * correctly, in order to give accurate results.
 */
int main(int argc, char *argv[])
{
  (void)argc;
  (void)argv;

  return 0;
}
